/***
 * Simple/pseudo jQuery
 * @author	RubaXa	<trash@rubaxa.org>
 */

(function (window, undef){
	var
		  document	= window.document

		, _push		= [].push
		, _slice	= [].slice

		, _rdot		= /\./g
		, _rnum		= /^\d+/
		, _rspace	= /\s+/
		, _rclass	= /[\r\n\t]/g
		, _rquote	= new RegExp('(^|[^\\\\])((\\\\)*)"', 'g')
	
		, trimLeft = /^\s+/
		, trimRight = /\s+$/

		, length = 'length'
		, prototype = 'prototype'
		, trim = String[prototype].trim

		, replace = 'replace'
		, classAttr = 'className'
		, getElement = 'getElement'
		, getElementById = getElement + 'ById'
		, getElementsByClassName = getElement + 'sByClassName'
		, getElementsByTagName = getElement + 'sByTagName'

		, querySelectorAll = 'querySelectorAll'

		, handlers	= {}
		, EventListener = 'EventListener'
		, addEventListener = 'add' + EventListener
		, removeEventListener = 'remove' + EventListener

		, querySelectorSupport = !!document[querySelectorAll]

		, cssNumber = {
				"fillOpacity": true,
				"fontWeight": true,
				"lineHeight": true,
				"opacity": true,
				"orphans": true,
				"widows": true,
				"zIndex": true,
				"zoom": true
			}
	;



	//
	// ajs.$
	//
	var $ = function (sel, context){
		if( !(this instanceof $) ){
			return	new $(sel, context);
		}
		else if( sel instanceof $ ){
			return	sel;
		}

		this[length] = 0;

		if( sel === 'body' ){
			this[0] = document.body;
			this[length] = 1;
		}
		else if (sel === window){
			this[0] = window;
			this[length] = 1;
		}
		else if( typeof sel === 'string' ){
			this.context	= context = (context || document);

			if( sel.charAt(0) == '<' ){
				var div	= document.createElement('div');
				div.innerHTML	= sel;
				this.push(div.childNodes);
				div = null;
			}
			else {
				this.push(find($(context), sel));
			}
		}
		else if( sel ) {
			if( sel.nodeType == 1 || sel.nodeType == 9 ){
				this[0] = sel;
				this[length] = 1;
			}
			else {
				this.push(sel);
			}
		}

		return	this;
	};


	$.gid	= 1;
	$.each	= function (elms, fn){
		for( var i = 0, n = elms[length]; i < n; i++ ){
			if( i in elms ){
				fn.call(elms[i], i, elms[i]);
			}
		}
	};

	$['extend']	= function (dst){
		var args = arguments, i = 1, n = args[length], val, key, src;

		if( !n ){
			$.expand($.fn, dst);
			return	this;
		}
		else {
			for( ; i < n; i++ ){
				if( src = args[i] ){
					for( key in src ) if( ((val = src[key]) !== null) && (val !== undef) ){
						dst[key] = val;
					}
				}
			}
			return	dst;
		}
	};

	$.noop	= function (){};
	$.quote = function (str){ return (str+'').replace(_rquote, '\"'); };
	$.trim	= trim
				? function( text ) { return text == null ? "" : trim.call( text ); }
				: function( text ) { return text == null ? "" : text.toString()[replace]( trimLeft, "" )[replace]( trimRight, "" ); }
			;


	$[prototype] = $.fn = {

		constructor: $,

		attr: function (name, val){
			var elm = this[0], i = 0, n = this[length], key, ret;

			if( typeof name == 'string' && val === undef ){
				if( elm ){
					if( (name in elm) || !('getAttribute' in elm) ) {
						ret	= elm[name];
					}
					else {
						ret	= elm.getAttribute(name);
					}
				}
				return	ret;
			}
			else {
				name	= objByKeyVal(name, val);
				for( var i = 0, n = this[length]; i < n; i++ ){
					elm	= this[i];
					if( elm ){
						for( key in name ){
							val	= name[key];
							if( key in elm || !('setAttribute' in elm) ){
								elm[key]	= val
							}
							else {
								elm.setAttribute(key, val);
							}
						}
					}
				}

				return	this;
			}
		},

		css: function (name, val){
			var elm = this[0], i = 0, n = this[length], key, ret;

			if( typeof name == 'string' && val === undef ){
				ret	= elm && elm.style && elm.style[name] || undef;
				if( _rnum.test(ret) ){
					ret	= parseFloat(ret);
				}
				return	ret
			}
			else {
				name	= objByKeyVal(name, val);
				for( var i = 0, n = this[length]; i < n; i++ ){
					elm	= this[i];
					if( elm && (elm = elm.style) ){
						for( key in name ){
							val	= name[key];

							if( val !== "" && !isNaN(val) && !cssNumber[key] ){
								val	+= 'px';
							}

							elm[key]	= val;
						}
					}
				}
			}

			return	this;
		},

		size: function (){
			return	this[length];
		},

		push: function (array, prev){

			if( prev ){
				var ret			= $(array);
				ret.prevObject	= this;
				return	ret;
			}

			for( var i = 0, n = array[length]; i < n; i++ ){
				this[i]	= array[i];
				this[length]++;
			}

			return	this;
		},

		each: function (fn){
			$.each(this, fn);
			return	this;
		},

		eq: function (idx){
			return	this.push(this[idx], 1);
		},

		filter: function (sel){
			sel	= sel.split('.');
			return	this.push(matchesSelector(this, sel[0], sel[1]), 1);
		},

		find: function (sel){
			return	this.push(find(this, sel), 1);
		},

		children: function (idx){
			if( this[0] ){
				var childs	= this[0].childNodes;
				return	this.push(idx === undef ? childs : childs[idx], 1);
			}
			return	this.push(0, 1);
		},

		end: function(){
			return	this.prevObject || $();
		},

		parent: function (){
			return	this.push(this[0] && this[0].parentNode, 1);
		},

		html: function (html){
			return	html === undef ? this[0] && this[0].innerHTML : this.each(function (){ this.innerHTML = html; });
		},

		val: function (val){
			if( val === undef ){
				return	this[0] && this[0].value || '';
			}
			else {
				return	this.each(function (){ this.value = val; });
			}
		},

		append: function (content){
			var elm = this[0];
			if( elm ){
				$(content).each(function (){ elm.appendChild(this); });
			}
			return	this;
		},

		bind: function (events, fn){
			return	eventListener(this, 1, events, fn);
		},

		unbind: function (events, fn){
			return	eventListener(this, 0, events, fn);
		},

		trigger: function (events){
			return	this.each(function (){
				try { this[events](); } catch (e){ console.log(this, events) }
			});
		},

		one: function (events, fn){
			return	this.bind(events, function l(){
				$(this).unbind(events, l);
				return	fn.apply(this, arguments);
			}.bind(this));
		},

		hasClass: function (val){
			val	= val[replace](_rclass, ' ').split(_rspace);

			for ( var i = 0, l = this[length]; i < l; i++ ) {
				if( matchClasses(this[i], val) ) {
					return true;
				}
			}

			return false;
		},

		addClass: function (val){
			val	= val[replace](_rclass, ' ').split(_rspace);

			for ( var i = 0, l = this[length], elm, className, c = 0; i < l; i++ ) {
				elm	= this[i];

				if( elm.nodeType == 1 ){
					if( elm[classAttr] ){
						className = (" "+elm[classAttr]+" ")[replace](_rclass);

						for( c = val[length]; c--;  ){
							if( !~className.indexOf(" "+val[c]+" ") ){
								className	+= val[c]+" ";
							}
						}

						elm[classAttr]	= $.trim(className);
					}
					else {
						elm[classAttr]	= val.join(' ');
					}
				}
			}

			return	this;
		},

		removeClass: function (val){
			val	= val[replace](_rclass, ' ').split(_rspace);

			for ( var i = 0, l = this[length], elm, className, c = 0; i < l; i++ ) {
				elm	= this[i];
				if( elm.nodeType == 1 && elm[classAttr] ){
					className = (" "+elm[classAttr]+" ")[replace](_rclass);

					for( c = val[length]; c--;  ){
						className	= className[replace](" "+val[c]+" ", " ");
					}

					elm[classAttr]	= $.trim(className);
				}
			}

			return this;
		},

		toggleClass: function (className, state){
			return	this[(state === undef ? !this.hasClass(className) : state) ? 'addClass' : 'removeClass'](className);
		},

		closest: function (sel){
			var elm = this[0], tag;

			if( elm ){
				sel	= sel.split('.');
				tag	= sel[0] ? sel[0].toUpperCase() : false;
				sel	= sel[1] ? sel.slice(1) : false;

				do {
					if( (elm.nodeType == 1) && (!tag || elm.tagName.toUpperCase() == tag) && (!sel || matchClasses(elm, sel)) ){
						return	this.push(elm, 1);
					}
				}
				while( elm && (elm = elm.parentNode) && (elm !== document.body) );
			}

			return	this.push(undef, 1);
		},

		show: function (){
			return	this.css('display', '');
		},

		hide: function (){
			return	this.css('display', 'none');
		},

		toggle: function (state){
			return	this[state ? 'show' : 'hide']();
		},

		remove: function (){
			return	this.each(function (){ this.parentNode && this.parentNode.removeChild(this); });
		}

	};


	$.get = function (url, complete){
		var
			  callback = 'ajs__jsonp__' + $.gid++
			, transport = document.createElement('script')
		;
		
		transport.type	= 'text/javascript';
		transport.src	= url = url+'&callback='+callback;

		window[callback] = function (result){
			complete(result);
			$(transport).remove();
			window[callback] = null;
			try { delete window[callback]; } catch (er){}
		};

		$('head').append(transport);
	};


	function matchClasses(elm, classList){
		if( elm.nodeType == 1 ){
			var className	= (" "+ elm[classAttr] +" ")[replace](_rclass, ' ');

			for( var i = 0, n = classList[length]; i < n; i++ ) {
				if( !~className.indexOf(" "+classList[i]+" ") ) {
					return	false;
				}
			}

			return true;
		}
	}




	if( Function[prototype].bind === undef ) Function[prototype].bind = function (context) {
		var args = _slice.call(arguments, 1), self = this;
		return function (){ return self.apply(context, args.concat(_slice.call(arguments, 0))); };
	};


	function eventListener(elms, bind, events, fn){
		events	= events.split(_rspace);

		var j = elms[length], elm, i, n = events[length], evt, key, handler;

		fn.gid	= fn.gid || $.gid++;
		for( ; j--; ){
			if( elm = elms[j] ){
				elm.ajs	= elm.ajs || $.gid++;

				for( i = 0; i < n; i++ ){
					evt		= events[i];
					key		= elm.ajs +'_'+ evt +'_'+ fn.gid;
					handler	= handlers[key] = (handlers[key] || function (event){
						event	= event || window.event;

						if( !event.target ){
							event.target	= event.srcElement || document;
						}

						if ( event.target.nodeType === 3 ) {
							event.target = event.target.parentNode;
						}

						if( !event.currentTarget ){
							event.currentTarget	= elm;
						}

						if ( !event.relatedTarget && event.fromElement ) {
							event.relatedTarget = event.fromElement === event.target ? event.toElement : event.fromElement;
						}

						if( !event.stopPropagation ){
							event.stopPropagation	= function () { this.cancelBubble = true; };
						}

						if( !event.preventDefault ){
							event.preventDefault	= function () { this.returnValue = false; };
						}

						if( fn.call(elm, event) === false ){
							event.preventDefault();
							event.stopPropagation();
							return	false;
						}
					});

					if( elm[addEventListener] ){
						elm[bind ? addEventListener : removeEventListener](evt, handler, false);
					}
					else {
						elm[bind ? 'attachEvent' : 'detachEvent']('on'+evt, handler);
					}
				}
			}
		}

		return	elms;
	}


	function objByKeyVal(key, val){
		if( val !== undef ){
			var obj		= {};
			obj[key]	= val;
			return	obj;
		}

		return	key;
	}

	var findBySelector = /\[native\s+code\]/i.test(document[getElementsByClassName])
		? function (classList, context){ return context[getElementsByClassName](classList); }
		: function (classList, context){ return matchesSelector(context[getElementsByTagName]('*'), null, classList); }
	;


	function find(context, selector){
		var
			  res, ret = []
			, c = 0
			, cn = context[length]
			, p = selector.indexOf(' ')
			, sel = (~p ? selector.substr(0, p) : selector)[replace](_rspace)
			, prefix, _sel, ctx, tmp
		;

		for( ; c < cn; c++ ){
			ctx	= context[c];

			if( ~p && querySelectorSupport ){
				res	= ctx[querySelectorAll](selector);
			}
			else {
				_sel	= sel.substr(1);
				prefix	= sel.charAt(0);

				if( prefix === '#' ){
					res	= document[getElementById](_sel);
					res	= res ? [res] : [];
				}
				else {
					if( prefix === '.' ){
						res	= findBySelector(_sel[replace](_rdot, ' '), ctx);
					}
					else {
						tmp	= sel.split('.');
						res	= ctx[getElementsByTagName](tmp[0]);
						if( ~sel.indexOf('.') ){
							res	= matchesSelector(res, null, tmp.slice(1));
						}
						else {
							res	= toArray(res);
						}
					}
				}
			}

			if( res[length] ){
				if( ~p && !querySelectorSupport ){
					res	= find(res, selector.substr(p+1));
				}

				ret	= ret.concat(_slice.call(res, 0));
			}
		}

		return	ret;
	}


	function matchesSelector(elms, tag, classList){
		var ret	= [], i = 0, n = elms[length], elm;

		if( tag ){
			tag	= tag.toUpperCase();
		}

		if( classList && (typeof classList == 'string') ){
			classList	= classList[replace](_rdot, ' ').split(_rspace);
		}

		for( ; i < n; i++ ){
			elm		= elms[i];
			if( (elm.nodeType == 1) && (!tag || (tag == elm.tagName.toUpperCase())) && (!classList || matchClasses(elm, classList)) ){
				ret.push(elm);
			}
		}

		return	ret;
	}


	function toArray(arr){
		var ret = [], i = 0, n = arr.length;
		for( ; i < n; i++ ) if( i in arr ){
			ret[i]	= arr[i];
		}
		return	ret;
	}



	if( !('ajs' in window) ){
		window.ajs = {};
	}

	if( !('ui' in ajs) ){
		ajs.ui	= {};
	}

	window.ajs.$ = $;
})(this);
/**
 * @class	ajs.ui.dropdown
 *

	<div class="js-dropdown">
		<div class="js-link">click me</div>
		<ul class="js-menu" style="display: none">
			<li>item 1</li>
			<li>item 2</li>
			<li>item 3</li>
		</ul>
	</div>

	<script type="text/javascript">
		 ajs.ui.dropdown('.js-dropdown', {
			  cnLink: 'js-link'
			, cnMenu: 'js-menu'
			, cnExpand: 'js-expand'
			, timeout: 1500
 			, hideByClick: true
		});
	</script>
 */
(function (document, $, undef){
	function dropdown(sel, opts) {
		sel	= $(sel);

		if( sel.size() > 1 ){
			sel.each(function (){ dropdown(this, opts); });
			return	sel;
		}

		if( !(this instanceof dropdown) ){
			return	new dropdown(sel, opts);
		}

		this.opts	= opts = $.extend({
						  cnLink:       'js-link'
						, cnMenu:       'js-menu'
						, cnExpand:     'js-expand'
						, cnHover:      'js-hover'
						, onselect:     $.noop
						, timeout:      1500
						, hideByClick:  true
					}, opts);

		// Delegate
		this.__event = this._event.bind(this);

		// Container
		this.$Elm	= sel;

		// Link
		this.$Link	= sel.find('.'+opts.cnLink)
						.bind('click mouseout mouseover blur', this.__event)
						.attr({ tabIndex: ajs.ui.tabIndex++, unselectable: 'off' })
						.css({ 'userSelect': 'none', 'MozUserSelect': 'none', 'WebKitUserSelect': 'none', 'OUserSelect': 'none', 'MSUserSelect': 'none' })
					;

		// IE < 10
		this.$Link.find('*').attr({ unselectable: 'off' });

		// Menu
		this.$Menu	= sel.find('.'+opts.cnMenu).bind('click mouseout mouseover mousedown mouseup', this.__event);

		$(document).bind('keyup', function (evt){
			var key = evt.keyCode;
			if( (key == 40 || key == 38 || key == 13) && this.expanded() ){
				if( key == 13 ){
					this._onSelect(evt);
					this._expand(false);
				} else {
					this._move(key == 40 ? +1 : -1);
				}
			}
			else if( (key == 13 || key == 40) && evt.target === this.$Link[0] ){
				this._event(evt, 'click');
			}
		}.bind(this));
	}

	dropdown.prototype = {
		constructor: dropdown,

		_onSelect: function (evt, node){
			if( node || (node = this.$Menu.find('.'+this.opts.cnHover)[0]) ){
				this.opts.onselect.call(this, evt, node);
			}
		},

		_move: function (direction){
			var
				  $Items = this.$Menu.find('.js-section')
				, cnHover = this.opts.cnHover
				, idx = -1
			;


			$Items.each(function (i, node){
				node = $(node);
				if( node.hasClass(cnHover) ){
					idx = i;
					node.removeClass(cnHover);
				}
			});


			if( direction ){
				if( direction.tagName ){
					$(direction).addClass(cnHover);
				} else {
					if( idx != -1 ) idx += direction;
					if( !$Items[idx] ) idx = ~direction ? 0 : $Items.length-1;
					$Items.eq(idx).addClass(cnHover);
				}
			}
		},

		_event: function (evt, type){
			var
				  opts = this.opts
				, target = evt.currentTarget
				, isExpand = this.expanded()
			;

			if( type === undef ){
				type = evt.type;
			}

			clearTimeout(this._tid);

			if( type == 'mouseup' ){
				this.__down = false;
			} else if( type == 'mousedown' ){
				this.__down = true;
			} else if( type == 'blur' && !this.__down ){
				target  = evt.explicitOriginalTarget || document.activeElement;
				if( target.nodeType != 1 ) target = target.parentNode;
				if( !this._hitTest(target) ){
					this._expand(false);
				}
			}
			else if( type == 'click' ){
				if( target === this.$Link[0] ){
					evt.preventDefault();
				} else {
					this._onSelect(evt);
				}

				if( opts.hideByClick || !this._hitTest(evt) ){
					this._expand(!isExpand);
				}
			}
			else if( type == 'mouseover' ){
				// clear hover
				target  = evt.target;
				do {
					if( target.className.indexOf('js-section ') != -1 ){
						this._move(target);
						break;
					}
				} while( target !== this.$Menu[0] && target !== this.$Link[0] && (target = target.parentNode) );
			}
			else if( (opts.timeout > 0) && (type == 'mouseout') ){
				this._tid	= setTimeout(this._expand.bind(this, false), opts.timeout);
			}
		},

		_hitTest: function (evt){
			return	!!$(evt.target || evt).closest('.'+this.opts.cnMenu).size();
		},

		_expand: function (state){
			this.$Elm.toggleClass(this.opts.cnExpand, state);
			this.$Menu.toggle(state);

			$(document).unbind('click', this.__event);

			if( state ){
				this._move(); // clear hover
				setTimeout(function (){ $(document).bind('click', this.__event); }.bind(this), 0);
			}
		},

		expanded: function (){
			return  this.$Elm.hasClass(this.opts.cnExpand);
		}

	};


	ajs.ui.tabIndex = ajs.ui.tabIndex || 1;
	ajs.ui.dropdown	= dropdown;
})(this.document, ajs.$);
/**
 * @class	ajs.ui.search
 * @require	ajs.ui.dropdown, ajs.ui.suggest
 *

	<div class="js-search">
		<form class="js-from">
 			<input type="submit" value="search" />
			<label for="uniqId">text...</label>
			<input class="js-input" id="uniqId" type="text" />
			<div>
				<div class="js-section-text js-section-link">mail.ru</div>
				<ul class="js-section-menu" style="display: none">
					<li><a href="action-url" class="js-section">mail.ru</a></li>
					<li><a href="action-url" class="js-section">win.mail.ru</a></li>
					<li><a href="action-url" class="js-section">games.mail.ru</a></li>
				</ul>
			</div>

		</form>
	</div>
	<script type="text/javascript">
		ajs.ui.search('.js-search', {
			  cnForm: 'js-form'
			, cnInput: 'js-input'
			, cnLabel: 'js-label'
			, cnBlur: 'js-blur'
			, cnSectionTxt: 'js-section-text'
			, cnSectionLink: 'js-section-link'
			, cnSectionMenu: 'js-section-menu'
 			, cnSectionExpand: 'js-section-expand'
 			, cnSectionSelected: 'js-selected'
			, cnSection: 'js-section'
			, paddingRight: 35
			, suggest: {  } // options
		});
	</script>
 */



(function ($, undef){

	function search(sel, opts){
		sel	= $(sel);

		if( sel.size() > 1 ){
			sel.each(function (){ search(this); });
			return	sel;
		}

		if( !(this instanceof search) ){
			return	new search(sel, opts);
		}
		

		this.opts	= opts = $.extend({
						  cnForm:		'js-form'
						, cnInput:		'js-input'
						, cnBlur:		'js-blur'
						, cnSection:	        'js-section'
						, cnSectionTxt:	        'js-section-text'
						, cnSectionLink:        'js-section-link'
						, cnSectionMenu:        'js-section-menu'
						, cnSectionHover:       'js-section-hover'
						, cnSectionExpand:      'js-section-expand'
						, cnSectionSelected:    'js-section-selected'
						, paddingRight:	35
						, searchInputResize: true
					}, opts);

		// Delegate
		this.__event	= this._event.bind(this);

		// Container
		this.$Elm		= sel;

		// Form
		this.$Form		= sel.find('.'+opts.cnForm).bind('submit', this.__event);

		// Input
		this.$Input		= sel.find('.'+opts.cnInput)
							.bind('focus blur', this.__event)
							.attr({ tabIndex: ajs.ui.tabIndex++ })
						;

		// Current section
		this.$Section	= sel.find('.'+opts.cnSectionTxt);

		// Init Sesctions
//		this.$Elm.find('.'+opts.cnSectionMenu).bind('click', this._section.bind(this));

		// Section Menu (dropdown)
		if( ajs.ui.dropdown ){
			ajs.ui.dropdown(sel, {
				  cnLink:   opts.cnSectionLink
				, cnMenu:   opts.cnSectionMenu
				, cnHover:  opts.cnSectionHover
				, cnExpand: opts.cnSectionExpand
				, onselect: this._section.bind(this)
			});
		}

		// Set tabIndex for search btn
		sel.find('input').each(function (){
			if( this.type == 'submit' ){
				this.tabIndex = ajs.ui.tabIndex++;
			}
		});

		// Suggest
		if( opts.suggest && ajs.ui.suggest ){
			this.Suggest	= ajs.ui.suggest(sel, opts.suggest);
		}

		// Init
		this._event({ type: 'blur' });
		this._section({ /* >__< */ });
		this.isReady = true;
	}

	search.prototype	= {
		constructor:	search,

		_event: function (evt){
			var type = evt.type, isEmpty = this.isEmpty(), isBlur = (type == 'blur' && isEmpty);

			if( type == 'submit' && isEmpty ){
				this.$Input.trigger('focus');
				evt.preventDefault();
			}
			else {
				this.$Elm.toggleClass(this.opts.cnBlur, isBlur);
				if( isBlur ){
					this.val('');
				}
			}
		},

		_section: function (evt, node){
			var
				  opts = this.opts
				, target = node || evt.target
				, cnSelected = opts.cnSectionSelected
				, $Item = $(target).closest('.'+opts.cnSection)
				, $Sections = this.$Elm.find('.'+opts.cnSection)
			;

			if( target ){
				$Sections.filter('.'+cnSelected).removeClass(cnSelected);
				$Item.addClass(cnSelected);
			}

			if( ($Item = $Sections.filter('.'+cnSelected))[0] ){
				// Set form action
				this.$Form.attr({
					  'action':	$Item.attr('data-action')
					, 'target':	$Item.attr('data-target') || '_self'
				});
				this.$Input.attr('name', $Item.attr('data-input-name'));

				// Replace html
				this.$Section.html($Item.html());

				if( this.Suggest ){
					this.Suggest.set('ajaxUrl', $Item.attr('data-suggest-url'), !this.isReady);
				}
			}

			if( evt && evt.preventDefault ){
				if( evt.type == 'click' || evt.keyCode == 13 ){
					this.$Input.trigger('focus');
				}
				evt.preventDefault();
			}
		}

		, isEmpty: function (){
			return	this.val() == "";
		},

		val: function (val){
			if( val !== undef ){
				this.$Input.val(val);
				return	this;
			}
			return	$.trim(this.$Input.val());
		}
	};


	ajs.ui.tabIndex = ajs.ui.tabIndex || 1;
	ajs.ui.search	= search;
})(ajs.$);
/**
 * @class	ajs.ui.suggest
 *

	<form class="js-suggest-box">
		<input class="js-input" type="text" />
		<div class="js-suggest" style="display: none">
			<div class="js-suggest-list"></div>
			<a href="" class="js-suggest-internet"></a>
		</div>
	</form>
	<script type="text/javascript">
	 ajs.ui.suggest('.js-suggest-box', {
		  cnInput: 'js-input'
		, cnSuggest: 'js-suggest'
		, cnList: 'js-suggest-list'
		, cnItem: 'js-suggest-item'
		, cnSelected: 'js-suggest-selected'
		, cnInternet: 'js-suggest-internet'
        , searchLast: false
 		, getData: function (res, val){ return res && res[1] == 'OK' && res[2]; }
		, template: function (val, item){ return '<div data-suggest="'+ajs.$.quote(item)+'" class="'+this.cnItem+'">'+item.replace(new RegExp(val,'i'), '<b>'+item.match(new RegExp(val,'i'))+'</b>')+'</div>'; }
 		, request: function (url, fn, val){ ajs.$.get(url, fn); }
		, timeout: 300
        , minLength: 1
        , maxLength: 0
 		, expire: 0
 		, searchLast: true
		, autosubmit: true
		, ajaxUrl: 'http://f6.mail.ru/cgi-bin/gosearch_ajax?ajax_call=1&func_name=ajax_suggest&data=["#query#"]'
		, searchUrl: 'http://go.mail.ru/search?mailru=1&rch=l&q=#query#'
 		, onselect: function (suggestEvent){ return false; }
	 });

	//
 	//	suggestEvent = {
	//		  type:		'select'
	//		, target:	node
	//		, event:	evt  // original event
	//		, item:		{ ... }  // selected suggest item
	//		, value:	val // selected value
	//		, suggest:	suggestInstance
	//	}
	//
	</script>
 */


(function ($, undef){
	var
		  _lastSpace = /^.*\s+/
		, _notSpace = /\S+$/
		, encode = encodeURIComponent
	;


	function suggest(sel, opts){
		sel	= $(sel);

		if( sel.size() > 1 ){
			sel.each(function (){ suggest(this); });
			return	sel;
		}


		if( !(this instanceof suggest) ){
			return	new suggest(sel, opts);
		}

		this.opts	= opts = $.extend({
						  cnInput:		'js-input'    // input for text
						, cnSuggest:	'js-suggest'  // parent container
						, cnList:		'js-suggest-list' // container for items
						, cnItem:		'js-suggest-item'  // suggest item
						, cnSelected:	'js-suggest-selected' // selected suggest item
						, cnInternet:	'js-suggest-internet' // element for "Search in internet"
						, getData:		function (res){ return res && res[1] == 'OK' && res[2]; } // processing ajax result, get suggest items (array)
						, template:		function (val, item){ return '<div data-suggest="'+$.quote(item)+'" class="'+this.cnItem+'">'+item.text.replace(new RegExp(val,'i'), '<b>$&</b>')+'</div>'; } // item example
						, request:		function (url, fn, val){ $.get(url, fn); } // ajax/jsonp request for get suggest list (+ see. getData)
						, searchLast:   false
						, minLength:    1 // min characters
						, maxLength:    0 // max characters (0 � Infinity)
						, timeout:		300 // search "suggests" after XXXms and after keyup
						, expire:		0	// expire local cache after XXXs
						, autosubmit:	true	// submit form after click or enter by suggest item
						, ajaxUrl:		undef	// ajax/jsonp url for receive "suggest list"
						, searchUrl:	'http://go.mail.ru/search?q=#query#' // a.href for cnInternet element
						, onselect:		$.noop
					}, opts);

		// Cache data
		this._cache		= {};

		// Container
		this.$Elm		= sel;

		// Input
		this.$Input		= sel.find('.'+opts.cnInput).attr('autocomplete', 'off').bind('blur focus click keyup keydown', this._event.bind(this));

		// Suggest
		this.$Suggest	= sel.find('.'+opts.cnSuggest)
							.bind('click', this._onClick.bind(this))
							.bind('mouseup mousedown', function (evt){ this._down = evt.type != 'mouseup'; }.bind(this))
						;

		// Search in internet
		this.$Internet	= sel.find('.'+opts.cnInternet);

		// Suggest List
		this.$List		= sel.find('.'+opts.cnList).bind('click', this._selectByClick.bind(this));

		// Expire suggest cache
		if( opts.expire ) setInterval(this.reset.bind(this), opts.expire*1000);
	}

	suggest.prototype	= {
		constructor:	suggest,

		_event: function (evt){
			var
				  type = evt.type
				, key = evt.keyCode
				, opts = this.opts
			;

			clearTimeout(this._tid);

			if( type == 'blur' ){
				this.blurred = true;
				if( !this._down ){
					this.$Suggest.hide();
				}
			} else {
				if( type == 'focus' ){
					this.blurred = false;
				}

				if( key == 38 || key == 40 ){
					// 38 - Up, 40 - Down
					if( type == 'keydown' ){
						this._select(key == 38 ? -1 : 1);
					}
					return	false;
				}
				else if( key == 13 && type == 'keydown' ){
					var $Sel = this.$List.find('.'+opts.cnSelected);
					if( $Sel[0] && (this._onselect(evt, $Sel[0]) === false) ){
						return	false;
					}
				} else {
					var val = this.val();

					if( opts.maxLength > 0 ){
						val = val.substr(0, opts.maxLength);
					}

					if( this._cache[val] ){
						this._rebuild(val);
					}
					else if( this.val.length >= opts.minLength && this._checkPrevResult(val, opts) ){
						this._tid = setTimeout(this.find.bind(this, val), opts.timeout);
					}
				}
			}
		},

		_checkPrevResult: function (val, opts){
			var i = val.length-1, key, cache = this._cache;

			for( ; i >= opts.minLength ; i-- ){
				key = val.substr(0, i);
				if( key in cache )
					return  cache[key].length > 0;
			}

			return  true;
		},

		_onClick: function (){
			this._event({ type: 'blur' });
		},

		_closest: function (node, val){
			do {
				if( (node.nodeType == 1) && ('getAttribute' in node) ){
					val	= node.getAttribute('data-suggest-idx');
					if( val > 0 ){
						return	node;
					}
				}
			}
			while( node && (node = node.parentNode) && (node !== document.body) );
		},

		_selectByClick: function (evt){
			var
				  opts = this.opts
				, sel = opts.cnSelected
				, $Item = $(this._closest(evt.target))
				, val = $Item.attr('data-suggest')
			;

			if( this._onselect(evt, $Item[0]) !== false ){

				this.$List.find('.'+sel).removeClass(sel);
				this.$Input.trigger('focus');
				this.val( val );

				if( opts.autosubmit ){
					var $Inp = this.$Input.closest('form').find('input'), i = $Inp.length;
					for( ; i--; ) if( $Inp[i].type == 'submit' ){
						$Inp[i].click();
						break;
					}
				}
			}

			return	false;
		},

		_select: function (move){
			var
				  sel = this.opts.cnSelected
				, $Sel = this.$List.find('.'+sel)
				, $List = this.$List
				, $childs = $List.children()
			;

			if( $Sel.size() ){
				$Sel.removeClass(sel);
				$Sel = $($Sel[0][move > 0 ? 'nextSibling' : 'previousSibling']).addClass(sel);
			}
			else {
				$Sel = $childs.eq(move > 0 ? 0 : $childs.size()-1).addClass(sel);
			}

			this.val( $Sel[0] ? $Sel.attr('data-suggest') : this.__val );
		},

		_loaded: function (val, result){
			this._cache[val] = this.opts.getData.call(this, result, val);
			if( this.val() == val && !this.blurred ){
				this._rebuild(val);
			}
		},

		_rebuild: function (val){
			var data = this._cache[val];

			if( this.__val !== val ){
				this.__val = val;

				if( data && data.length ){
					var i = 0, n = data.length, html = [];

					for( ; i < n; i++ ){
						html.push($.trim(this.opts.template.call(this, val, data[i])));
					}

					this.$List
						.html(html.join(''))
						.children()
							.each(function (i, N){
								N.setAttribute('data-suggest-idx', i + 1);
							})
					;
				}

				this._internetVal(val);
			}

			this.$List.toggle( !!(data && data.length) );
			this.$Suggest.toggle( !!(val.length > 1) );
		},
		
		_onselect: function (evt, node){
			var
				  val = node.getAttribute('data-suggest')
				, idx = node.getAttribute('data-suggest-idx')-1
				, cache = this._cache[this.__val]
			;

			return	this.opts.onselect({
				  type:		'select'
				, target:	node
				, event:	evt
				, item:		cache && cache[idx]
				, value:	val
				, suggest:	this
			})
		},

		_request: function (url, fn, val){
			this.opts.request(url, fn, val);
		},

		_internetVal: function (val){
			this.$Internet.attr({
				  'href': this.opts.searchUrl.replace('#query#', encode(val))
				, 'innerHTML': val
			});
		},
		

	// @public
		val: function (val){
			if( val !== undef ){
				/** @namespace this.opts.searchLast */
				if( this.opts.searchLast ){
					val = this.$Input.val().replace(_notSpace, '') + val;
				}

				this._internetVal(val);
				this.$Input.val(val);

				if( val == "" ){
					this.$Suggest.hide();
				}
				return	this;
			}

			val = $.trim(this.$Input.val());

			if( this.opts.searchLast ){
				val	= val.replace(_lastSpace, '');
			}

			return	val;
		},

		find: function (val){
			var url = this.opts.ajaxUrl;

			if( !url || val == "" ){
				this.$Suggest.hide();
			}
			else if( (this._loadVal != val) || (this._loadUrl != url) ){
				this._loadUrl = url;
				this._loadVal = val;
				this._request(url.replace('#query#', encode(val)) +'&'+ Math.random(), this._loaded.bind(this, val), val);
			}
		},

		set: function (name, val, force){
			this.opts[name] = val;
			if( !force ) this.find( this.val() );
			return	this;
		},

		get: function (name){
			return	this.opts[name];
		},

		reset: function (){
			var val = this._loadVal, cache = this._cache[val];

			this.__val  = this._loadVal = undef;
			this._cache = {};

			if( this.$Suggest.css('display') != 'none' && cache ){
				this._cache[val] = cache;
			}
		}

	};

	ajs.ui.suggest	= suggest;
})(ajs.$);
